@@ -2,7 +2,7 @@
 /*                                                                          */
 /*  The FreeType project -- a free and portable quality TrueType renderer.  */
 /*                                                                          */
-/*  Copyright 1996-2002, 2003, 2004, 2005, 2006, 2007, 2009 by              */
+/*  Copyright 1996-2007, 2009-2013 by                                       */
 /*  D. Turner, R.Wilhelm, and W. Lemberg                                    */
 /*                                                                          */
 /*                                                                          */
@@ -10,11 +10,9 @@
 /*                                                                          */
 /****************************************************************************/
 
-#include <ft2build.h>
-#include FT_FREETYPE_H
 
-#include "common.h"
 #include "ftcommon.h"
+#include "common.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -23,18 +21,23 @@
 #include <math.h>
 
 #define CELLSTRING_HEIGHT  8
-#define MAXPTSIZE  500                 /* dtp */
+#define MAXPTSIZE          500   /* dtp */
 
 
   static char*  Text = (char *)"The quick brown fox jumps over the lazy dog";
 
-  enum {
+  enum
+  {
     RENDER_MODE_STRING,
     RENDER_MODE_KERNCMP,
     N_RENDER_MODES
   };
 
-  static struct {
+  static struct  status_
+  {
+    int          width;
+    int          height;
+
     int          render_mode;
     FT_Encoding  encoding;
     int          res;
@@ -50,21 +53,22 @@
     char*        header;
     char         header_buffer[256];
 
-  } status = { RENDER_MODE_STRING, FT_ENCODING_UNICODE, 72, 48, 2.0, 0 };
+  } status = { DIM_X, DIM_Y,
+               RENDER_MODE_STRING, FT_ENCODING_UNICODE, 72, 48, 2.0, 0 };
 
   static FTDemo_Display*  display;
   static FTDemo_Handle*   handle;
 
 
-/****************************************************************************/
-/****************************************************************************/
-/****************************************************************************/
-/****                                                                    ****/
-/****                     E V E N T   H A N D L I N G                    ****/
-/****                                                                    ****/
-/****************************************************************************/
-/****************************************************************************/
-/****************************************************************************/
+  /*************************************************************************/
+  /*************************************************************************/
+  /*************************************************************************/
+  /****                                                                 ****/
+  /****                   E V E N T   H A N D L I N G                   ****/
+  /****                                                                 ****/
+  /*************************************************************************/
+  /*************************************************************************/
+  /*************************************************************************/
 
   static void
   event_help( void )
@@ -73,6 +77,7 @@
 
 
     FTDemo_Display_Clear( display );
+    grSetLineHeight( 10 );
     grGotoxy( 0, 0 );
     grSetMargin( 2, 1 );
     grGotobitmap( display->bitmap );
@@ -90,7 +95,6 @@
     grWriteln( "  b         : toggle embedded bitmaps (and disable rotation)" );
     grWriteln( "  f         : toggle forced auto-hinting" );
     grWriteln( "  h         : toggle outline hinting" );
-    grWriteln( "  l         : toggle low precision rendering" );
     grLn();
     grWriteln( "  1-2       : select rendering mode" );
     grWriteln( "  k         : cycle through kerning modes" );
@@ -114,7 +118,6 @@
     grWriteln( "  F7        : big rotate counter-clockwise" );
     grWriteln( "  F8        : big rotate clockwise" );
     grLn();
-    grLn();
     grWriteln( "press any key to exit this help screen" );
 
     grRefreshSurface( display->surface );
@@ -126,7 +129,7 @@
   event_font_change( int  delta )
   {
     if ( status.font_index + delta >= handle->num_fonts ||
-         status.font_index + delta < 0 )
+         status.font_index + delta < 0                  )
       return;
 
     status.font_index += delta;
@@ -173,10 +176,11 @@
 
 
   static void
-  event_gamma_change( double delta )
+  event_gamma_change( double  delta )
   {
-    int i;
-    double gamma_inv;
+    int     i;
+    double  gamma_inv;
+
 
     status.gamma += delta;
 
@@ -197,21 +201,21 @@
 
 
   static void
-  event_size_change( int delta )
+  event_size_change( int  delta )
   {
     status.ptsize += delta;
 
-    if ( status.ptsize < 1*64 )
-      status.ptsize = 1*64;
-    else if ( status.ptsize > MAXPTSIZE*64 )
-      status.ptsize = MAXPTSIZE*64;
+    if ( status.ptsize < 1 * 64 )
+      status.ptsize = 1 * 64;
+    else if ( status.ptsize > MAXPTSIZE * 64 )
+      status.ptsize = MAXPTSIZE * 64;
 
     FTDemo_Set_Current_Charsize( handle, status.ptsize, status.res );
   }
 
 
   static void
-  event_render_mode_change( int delta )
+  event_render_mode_change( int  delta )
   {
     if ( delta )
     {
@@ -237,7 +241,7 @@
   static int
   Process_Event( grEvent*  event )
   {
-    FTDemo_String_Context*  sc = &status.sc;
+    FTDemo_String_Context*  sc  = &status.sc;
     int                     ret = 0;
 
 
@@ -297,15 +301,6 @@
       FTDemo_Update_Current_Flags( handle );
       break;
 
-    case grKEY( 'l' ):
-      handle->low_prec = !handle->low_prec;
-      status.header    = handle->low_prec
-                         ? (char *)"rendering precision is now forced to low"
-                         : (char *)"rendering precision is now normal";
-
-      FTDemo_Update_Current_Flags( handle );
-      break;
-
     case grKEY( 'k' ):
       sc->kerning_mode = ( sc->kerning_mode + 1 ) % N_KERNING_MODES;
       status.header =
@@ -360,7 +355,7 @@
 
     case grKeyUp:       event_size_change(   64 ); break;
     case grKeyDown:     event_size_change(  -64 ); break;
-    case grKeyPageUp:   event_size_change(  640); break;
+    case grKeyPageUp:   event_size_change(  640 ); break;
     case grKeyPageDown: event_size_change( -640 ); break;
 
     case grKeyLeft:  event_angle_change(    -3 ); break;
@@ -383,6 +378,7 @@
     int       i, x, y;
     FT_Byte*  p = (FT_Byte*)bitmap->buffer;
 
+
     if ( bitmap->pitch < 0 )
       p += -bitmap->pitch * ( bitmap->rows - 1 );
 
@@ -413,19 +409,23 @@
       switch ( error_code )
       {
       case FT_Err_Ok:
-        sprintf( status.header_buffer, "%s %s (file `%s')", face->family_name,
+        sprintf( status.header_buffer,
+                 "%.50s %.50s (file `%.100s')", face->family_name,
                  face->style_name, basename );
         break;
+
       case FT_Err_Invalid_Pixel_Size:
-        sprintf( status.header_buffer, "Invalid pixel size (file `%s')",
+        sprintf( status.header_buffer, "Invalid pixel size (file `%.100s')",
                  basename );
         break;
+
       case FT_Err_Invalid_PPem:
-        sprintf( status.header_buffer, "Invalid ppem value (file `%s')",
+        sprintf( status.header_buffer, "Invalid ppem value (file `%.100s')",
                  basename );
         break;
+
       default:
-        sprintf( status.header_buffer, "File `%s': error 0x%04x", basename,
+        sprintf( status.header_buffer, "File `%.100s': error 0x%04x", basename,
             (FT_UShort)error_code );
         break;
       }
@@ -437,7 +437,7 @@
                        status.header, display->fore_color );
 
     sprintf( status.header_buffer, "at %g points, angle = %d",
-             status.ptsize/64.0, status.angle );
+             status.ptsize / 64.0, status.angle );
     grWriteCellString( display->bitmap, 0, CELLSTRING_HEIGHT,
                        status.header_buffer, display->fore_color );
 
@@ -448,17 +448,39 @@
   static void
   usage( char*  execname )
   {
-    fprintf( stderr,  "\n" );
-    fprintf( stderr,  "ftstring: string viewer -- part of the FreeType project\n" );
-    fprintf( stderr,  "-------------------------------------------------------\n" );
-    fprintf( stderr,  "\n" );
-    fprintf( stderr,  "Usage: %s [options below] ppem fontname[.ttf|.ttc] ...\n",
+    fprintf( stderr,
+      "\n"
+      "ftstring: string viewer -- part of the FreeType project\n"
+      "-------------------------------------------------------\n"
+      "\n" );
+    fprintf( stderr,
+      "Usage: %s [options] pt font ...\n"
+      "\n",
              execname );
-    fprintf( stderr,  "\n" );
-    fprintf( stderr,  "  -e enc      specify encoding tag (default: unic)\n" );
-    fprintf( stderr,  "  -r R        use resolution R dpi (default: 72 dpi)\n" );
-    fprintf( stderr,  "  -m message  message to display\n" );
-    fprintf( stderr,  "\n" );
+    fprintf( stderr,
+      "  pt        The point size for the given resolution.\n"
+      "            If resolution is 72dpi, this directly gives the\n"
+      "            ppem value (pixels per EM).\n" );
+    fprintf( stderr,
+      "  font      The font file(s) to display.\n"
+      "            For Type 1 font files, ftview also tries to attach\n"
+      "            the corresponding metrics file (with extension\n"
+      "            `.afm' or `.pfm').\n"
+      "\n" );
+    fprintf( stderr,
+      "  -w W      Set the window width to W pixels (default: %dpx).\n"
+      "  -h H      Set the window height to H pixels (default: %dpx).\n"
+      "\n",
+             DIM_X, DIM_Y );
+    fprintf( stderr,
+      "  -r R      Use resolution R dpi (default: 72dpi).\n"
+      "  -e enc    Specify encoding tag (default: no encoding).\n"
+      "            Common values: `unic' (Unicode), `symb' (symbol),\n"
+      "            `ADOB' (Adobe standard), `ADBC' (Adobe custom).\n"
+      "  -m text   Use `text' for rendering.\n"
+      "\n"
+      "  -v        Show version.\n"
+      "\n" );
 
     exit( 1 );
   }
@@ -476,7 +498,7 @@
 
     while ( 1 )
     {
-      option = getopt( *argc, *argv, "e:m:r:" );
+      option = getopt( *argc, *argv, "e:h:m:r:vw:" );
 
       if ( option == -1 )
         break;
@@ -487,9 +509,9 @@
         status.encoding = FTDemo_Make_Encoding_Tag( optarg );
         break;
 
-      case 'r':
-        status.res = atoi( optarg );
-        if ( status.res < 1 )
+      case 'h':
+        status.height = atoi( optarg );
+        if ( status.height < 1 )
           usage( execname );
         break;
 
@@ -499,6 +521,33 @@
         Text = optarg;
         break;
 
+      case 'r':
+        status.res = atoi( optarg );
+        if ( status.res < 1 )
+          usage( execname );
+        break;
+
+      case 'v':
+        {
+          FT_Int  major, minor, patch;
+
+
+          FT_Library_Version( handle->library, &major, &minor, &patch );
+
+          printf( "ftstring (FreeType) %d.%d", major, minor );
+          if ( patch )
+            printf( ".%d", patch );
+          printf( "\n" );
+          exit( 0 );
+        }
+        break;
+
+      case 'w':
+        status.width = atoi( optarg );
+        if ( status.width < 1 )
+          usage( execname );
+        break;
+
       default:
         usage( execname );
         break;
@@ -511,7 +560,7 @@
     if ( *argc <= 1 )
       usage( execname );
 
-    status.ptsize = (int)(atof( *argv[0] ) * 64.0);
+    status.ptsize = (int)( atof( *argv[0] ) * 64.0 );
     if ( status.ptsize == 0 )
       status.ptsize = 64;
 
@@ -527,11 +576,12 @@
     grEvent  event;
 
 
+    /* Initialize engine */
+    handle = FTDemo_New();
+
     parse_cmdline( &argc, &argv );
 
-    /* Initialize engine */
-    handle = FTDemo_New( status.encoding );
-
+    handle->encoding  = status.encoding;
     handle->use_sbits = 0;
     FTDemo_Update_Current_Flags( handle );
 
@@ -552,7 +602,8 @@
     if ( handle->num_fonts == 0 )
       PanicZ( "could not open any font file" );
 
-    display = FTDemo_Display_New( gr_pixel_mode_gray );
+    display = FTDemo_Display_New( gr_pixel_mode_gray,
+                                  status.width, status.height );
     display->back_color.value = 0;
     display->fore_color.value = 0xff;
 
@@ -589,8 +640,7 @@
 
           x = 55;
 
-          /* whatever.. */
-          height = status.ptsize * status.res / 72;
+          height = ( status.ptsize * status.res / 72 + 32 ) >> 6;
           if ( height < CELLSTRING_HEIGHT )
             height = CELLSTRING_HEIGHT;
 
@@ -607,7 +657,6 @@
                              "none", display->fore_color );
           error = FTDemo_String_Draw( handle, display, &sc, x, y );
 
-
           /* Second line: track kern only */
           sc.kerning_degree = status.sc.kerning_degree;
 
@@ -617,9 +666,8 @@
                              "track", display->fore_color );
           error = FTDemo_String_Draw( handle, display, &sc, x, y );
 
-
           /* Third line: track kern + pair kern */
-          sc.kerning_mode      = status.sc.kerning_mode;
+          sc.kerning_mode = status.sc.kerning_mode;
 
           y += height;
           grWriteCellString( display->bitmap, 5,
